"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseConfig = exports.pickUniqFieldsFromRecords = exports.pickUniqFieldsFromRecord = exports.mergeDefaultsWithPayload = void 0;
exports.validateInteger = validateInteger;
exports.idToTimestamp = idToTimestamp;
exports.idToDate = idToDate;
const utils_js_1 = require("../common/utils.js");
const constants_js_1 = require("./constants.js");
const errors_js_1 = require("./errors.js");
const mergeDefaultsWithPayload = async (schema, payload) => {
    const defaultPromises = Object.entries(schema).map(async ([key, prop]) => {
        if (prop.default &&
            typeof prop.default !== 'undefined' &&
            typeof payload[key] === 'undefined') {
            return {
                key,
                value: typeof prop.default === 'function' ? await prop.default() : prop.default
            };
        }
        else {
            return { key, value: undefined };
        }
    });
    const resolvedDefaults = await Promise.all(defaultPromises);
    const defaults = resolvedDefaults.reduce((acc, { key, value }) => {
        if (value !== undefined) {
            acc[key] = value;
        }
        return acc;
    }, {});
    return Object.assign(Object.assign({}, defaults), payload);
};
exports.mergeDefaultsWithPayload = mergeDefaultsWithPayload;
const pickUniqFieldsFromRecord = (schema, data) => {
    return Object.entries(data)
        .filter(([key]) => { var _a; return (_a = schema[key]) === null || _a === void 0 ? void 0 : _a.uniq; })
        .reduce((acc, [key, value]) => {
        if (key in schema) {
            acc[key] = value;
        }
        return acc;
    }, {});
};
exports.pickUniqFieldsFromRecord = pickUniqFieldsFromRecord;
const pickUniqFieldsFromRecords = (data, schema, label) => {
    const properties = {};
    const uniqFields = Object.entries(schema)
        .filter(([_, config]) => config.uniq)
        .reduce((acc, [key]) => {
        acc[key] = true;
        return acc;
    }, {});
    data.forEach((record) => {
        Object.entries(record).forEach(([key, value]) => {
            if (key in uniqFields) {
                if (properties[key]) {
                    if (properties[key].includes(value)) {
                        throw new errors_js_1.UniquenessError(label, { [key]: value });
                    }
                    properties[key] = [...properties[key], value];
                }
                else {
                    properties[key] = [value];
                }
            }
        });
    });
    return properties;
};
exports.pickUniqFieldsFromRecords = pickUniqFieldsFromRecords;
const parseConfig = (config) => {
    if (!config) {
        return {};
    }
    if (!(0, utils_js_1.isObject)(config)) {
        throw new Error('Config must be an object');
    }
    const values = Object.keys(config).filter((value) => !constants_js_1.ALLOWED_CONFIG_PROPERTIES.includes(value));
    if (values.length > 0) {
        throw new Error(`Config object may only contain the following: ${constants_js_1.ALLOWED_CONFIG_PROPERTIES.join(', ')}`);
    }
    return config;
};
exports.parseConfig = parseConfig;
function validateInteger(name, n, defaultVal) {
    if (!Number.isInteger(n)) {
        if (defaultVal !== undefined) {
            return defaultVal;
        }
        else {
            throw new Error(`${name} must be an integer`);
        }
    }
    return n;
}
function idToTimestamp(id) {
    const parts = id.split('-');
    const highBitsHex = parts[0] + parts[1].slice(0, 4);
    return parseInt(highBitsHex, 16);
}
function idToDate(id) {
    return new Date(idToTimestamp(id));
}
