import { isObject } from '../common/utils.js';
import { ALLOWED_CONFIG_PROPERTIES } from './constants.js';
import { UniquenessError } from './errors.js';
export const mergeDefaultsWithPayload = async (schema, payload) => {
    const defaultPromises = Object.entries(schema).map(async ([key, prop]) => {
        if (prop.default &&
            typeof prop.default !== 'undefined' &&
            typeof payload[key] === 'undefined') {
            return {
                key,
                value: typeof prop.default === 'function' ? await prop.default() : prop.default
            };
        }
        else {
            return { key, value: undefined };
        }
    });
    const resolvedDefaults = await Promise.all(defaultPromises);
    const defaults = resolvedDefaults.reduce((acc, { key, value }) => {
        if (value !== undefined) {
            acc[key] = value;
        }
        return acc;
    }, {});
    return Object.assign(Object.assign({}, defaults), payload);
};
export const pickUniqFieldsFromRecord = (schema, data) => {
    return Object.entries(data)
        .filter(([key]) => { var _a; return (_a = schema[key]) === null || _a === void 0 ? void 0 : _a.uniq; })
        .reduce((acc, [key, value]) => {
        if (key in schema) {
            acc[key] = value;
        }
        return acc;
    }, {});
};
export const pickUniqFieldsFromRecords = (data, schema, label) => {
    const properties = {};
    const uniqFields = Object.entries(schema)
        .filter(([_, config]) => config.uniq)
        .reduce((acc, [key]) => {
        acc[key] = true;
        return acc;
    }, {});
    data.forEach((record) => {
        Object.entries(record).forEach(([key, value]) => {
            if (key in uniqFields) {
                if (properties[key]) {
                    if (properties[key].includes(value)) {
                        throw new UniquenessError(label, { [key]: value });
                    }
                    properties[key] = [...properties[key], value];
                }
                else {
                    properties[key] = [value];
                }
            }
        });
    });
    return properties;
};
export const parseConfig = (config) => {
    if (!config) {
        return {};
    }
    if (!isObject(config)) {
        throw new Error('Config must be an object');
    }
    const values = Object.keys(config).filter((value) => !ALLOWED_CONFIG_PROPERTIES.includes(value));
    if (values.length > 0) {
        throw new Error(`Config object may only contain the following: ${ALLOWED_CONFIG_PROPERTIES.join(', ')}`);
    }
    return config;
};
export function validateInteger(name, n, defaultVal) {
    if (!Number.isInteger(n)) {
        if (defaultVal !== undefined) {
            return defaultVal;
        }
        else {
            throw new Error(`${name} must be an integer`);
        }
    }
    return n;
}
export function idToTimestamp(id) {
    const parts = id.split('-');
    const highBitsHex = parts[0] + parts[1].slice(0, 4);
    return parseInt(highBitsHex, 16);
}
export function idToDate(id) {
    return new Date(idToTimestamp(id));
}
